/*
 * Copyright (c) 2009-2013 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/types.h>
#include <errno.h>
#include <math.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#include <portLib.h>
#include <h2timeLib.h>

#include <ivmob/ivmob.h>

#include "ivmobConst.h"
#include "server/ivmobHeader.h"

#include "odoProba.h"

#ifdef GYRO
#include <gyroLib/gyro.h>
#endif

/*------------------------------------------------------------------------
 * Log
 *
 * Description:
 *
 * Reports:      OK
 *              S_rflex_CANNOT_OPEN_FILE
 *              S_rflex_CANNOT_WRITE_FILE
 */

static FILE *ivmobLogFile;

/* ivmobLogStart  -  codel START of Log
	Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
ivmobLogStart(GEN_STRING128 *fileName, int *report)
{
	if (!(ivmobLogFile = fopen(fileName->name, "a"))) {
		printf ("ivmobLogStart: cannot open odometer file %s\n",
		    fileName->name);
		*report = S_ivmob_CANNOT_OPEN_FILE;
		return ETHER;
	}
	if (!fprintf(ivmobLogFile, "date\tdate1\tdate2\tv\tw\tmain/yaw\tmain/pitch\tmain/roll\tmain/x\tmain/y\tmain/z\tvlocal/yaw\tvlocal/pitch\tvlocal/roll\tvlocal/x\tvlocal/y\tvlocal/z\n")) {
		printf ("ivmobLogStart: cannot log in %s\n", fileName->name);
		*report = S_ivmob_CANNOT_WRITE_FILE;
		ivmobLogInter(fileName, report);
		return ETHER;
	}
	printf ("ivmobLogStart: log file %s found\n", fileName->name);
	return EXEC;
}

/* ivmobLogExec  -  codel EXEC of Log
	Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
ivmobLogExec(GEN_STRING128 *fileName, int *report)
{
	H2TIMESPEC tv;

	/* get date */
	h2GetTimeSpec(&tv);
	if (!fprintf(ivmobLogFile, "%lf\t%d\t%d\t%2.6g\t%2.6g\t%2.6g\t%2.6g\t%2.6g\t%2.6g\t%2.6g\t%2.6g\t%2.6g\t%2.6g\t%2.6g\t%2.6g\t%2.6g\t%2.6g\n",
		tv.tv_sec + tv.tv_nsec*1.0E-9, SDI_F->mePos.date1, SDI_F->mePos.date2,
		SDI_F->robot.v, SDI_F->robot.w,
		SDI_F->mePos.main.euler.yaw, SDI_F->mePos.main.euler.pitch, SDI_F->mePos.main.euler.roll,
		SDI_F->mePos.main.euler.x, SDI_F->mePos.main.euler.y, SDI_F->mePos.main.euler.z,
		SDI_F->mePos.VLocal.euler.yaw, SDI_F->mePos.VLocal.euler.pitch, SDI_F->mePos.VLocal.euler.roll,
		SDI_F->mePos.VLocal.euler.x, SDI_F->mePos.VLocal.euler.y, SDI_F->mePos.VLocal.euler.z
		)) {
		printf ("ivmobLogExec: cannot log in %s\n", fileName->name);
		*report = S_ivmob_CANNOT_WRITE_FILE;
		ivmobLogInter(fileName, report);
		return ETHER;
	}
	return EXEC;
}


/* ivmobLogInter  -  codel INTER of Log
	Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
ivmobLogInter(GEN_STRING128 *fileName, int *report)
{
	if (ivmobLogFile) {
		fclose(ivmobLogFile);
		ivmobLogFile = NULL;
	}
	return ETHER;
}

/*------------------------------------------------------------------------
 * LogInternalData
 *
 * Description: Log all the raw data provided by robot
 *
 * Reports:     OK
 *              S_ivmob_CANNOT_OPEN_FILE
 *              S_ivmob_CANNOT_WRITE_FILE
 */

static FILE *ivmobLogInternalDataFile;

/* ivmobLogStart  -  codel START of Log
	Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
ivmobLogInternalDataStart(GEN_STRING128 *fileName, int *report)
{
	if (!(ivmobLogInternalDataFile = fopen(fileName->name, "a"))) {
		printf ("ivmobLogInternalDataStart: cannot open "
		    "internal data file %s\n", fileName->name);
		*report = S_ivmob_CANNOT_OPEN_FILE;
		return ETHER;
  }

	if (!fprintf(ivmobLogInternalDataFile,
		"date\tdate1\tdate2"
		"\tgyroAngle\tgyroRate"
		"\tsent_velocity_reference\tsent_velocity_command"
		"\tsent_turn_reference\tsent_turn_command"
		"\tstraight\tstraight_angle"
		/* begin of 4mob state */
		"\trs_mode\toperational_mode"
		"\tv\tw"
		"\tfront_left_wheel_speed\tfront_right_wheel_speed"
		"\trear_left_wheel_speed\t\rear_right_wheel_speed"
		"\tfront_left_current\tfront_right_current"
		"\trear_left_current\trear_right_current"
		"\tmmi_state\tsafety_code\n")) {
		printf ("ivmobLogStart: cannot log in %s\n", fileName->name);
		*report = S_ivmob_CANNOT_WRITE_FILE;
		ivmobLogInternalDataInter(fileName, report);
		return ETHER;
	}
	printf ("ivmobLogInternalDataStart: log file %s found\n", fileName->name);
	return EXEC;
}

/* ivmobLogInternalDataExec  -  codel EXEC of Log
	Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
ivmobLogInternalDataExec(GEN_STRING128 *fileName, int *report)
{
	H2TIMESPEC tv;
	IVMOB_STATUS_STR *status = &SDI_F->status;

	/* get date */
	h2GetTimeSpec(&tv);
	if (!fprintf(ivmobLogInternalDataFile, "%lf\t%d\t%d"
		"\t%.6g\t%.6g"
		"\t%.6g\t%.6g"
		"\t%.6g\t%.6g"
		"\t%d\t%.6g"
		/* start of feedback */
		"\t0x%08x\t0x%08x"
		"\t%.6g\t%.6g"
		"\t%.6g\t%.6g\t%.6g\t%.6g\t%.6g"
		"\t%.6g\t%.6g\t%.6g\t%.6g\t%.6g"
		"\t0x%08x\t0x%08x"
		"\n",
		/* timestamps */
		tv.tv_sec + tv.tv_nsec*1.0E-9,
		SDI_F->mePos.date1, SDI_F->mePos.date2,
		/* the KVH gyro */
		SDI_F->gyro.gyroTheta,
		SDI_F->gyro.gyroOmega,
		SDI_F->vReference, SDI_F->vCommand, // need to check if latest value is logged
		SDI_F->wReference, SDI_F->wCommand,
		SDI_F->gyroAsserv.straight, SDI_F->gyroAsserv.straight_angle,
		/* 4mob state */
		status->rs_mode,
		status->operational_mode,
		status->battery_level,
		status->temperature,
		status->v,
		status->w,
		status->front_left_wheel_speed,
		status->front_right_wheel_speed,
		status->rear_left_wheel_speed,
		status->rear_right_wheel_speed,
		status->front_left_current,
		status->front_right_current,
		status->rear_left_current,
		status->rear_right_current,
		status->mmi_state,
		status->safety_code))
	{
		printf ("ivmobLogInternalDataExec: cannot log in %s\n",
		    fileName->name);
		*report = S_ivmob_CANNOT_WRITE_FILE;
		ivmobLogInter(fileName, report);
		return ETHER;
	}
	return EXEC;
}

/* ivmobLogInternalDataInter  -  codel INTER of LogInternalData
	Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
ivmobLogInternalDataInter(GEN_STRING128 *fileName, int *report)
{
	if (ivmobLogInternalDataFile) {
		fclose(ivmobLogInternalDataFile);
		ivmobLogInternalDataFile = NULL;
	}
	return ETHER;
}
